% this is main code to
% do simulation for no bubble model
% six data six shock (measurement error)
% This version: May 14,2013

var Ct It SPt Qt Pt Nt ut Yt Xt Kt et grt gzt Lamt
    lam_zt lam_at v_at psit xit errt
    DLogC DLogI LogN DLogSP DLogY DLogZP FCt; 

varexo e_lam_zt e_lam_at e_v_at e_psit e_xit e_errt;

parameters alpha beta dele del del2 xi
           h BigOmg gamma K0K Growgr Growgz
           mu IY fc2 fc3
           rho_lam_z rho_lam_a rho_v_a rho_psi rho_xi 
           sig_lam_z sig_lam_a sig_v_a sig_psi sig_xi sig_err;

load Cali_Para
load Est_Para

set_param_value('alpha', alpha)
set_param_value('beta', beta )
set_param_value('del', del)
set_param_value('dele',dele)
set_param_value('Growgr',Growgr )
set_param_value('Growgz',Growgz )
set_param_value('h', h)
set_param_value('BigOmg',BigOmg )
set_param_value('del2', del2)
set_param_value('gamma',gamma )
set_param_value('mu',mu )
set_param_value('xi',xi)
set_param_value('IY',IY)
set_param_value('K0K',K0K)

set_param_value('rho_lam_z',rho_lam_z)
set_param_value('rho_lam_a',rho_lam_a)
set_param_value('rho_v_a',rho_v_a)
set_param_value('rho_psi',rho_psi)
set_param_value('rho_xi' ,rho_xi)

set_param_value('sig_lam_z',sig_lam_z)
set_param_value('sig_lam_a',sig_lam_a)
set_param_value('sig_v_a',sig_v_a)
set_param_value('sig_psi',sig_psi)
set_param_value('sig_xi' ,sig_xi)
set_param_value('sig_err',sig_err)

set_param_value('fc2',fc2)
set_param_value('fc3',fc3)

model(linear);

#gr   = Growgr/100+1; % 1+growth rate of output
#gz   = Growgz/100+1; % 1+growth rate of investment-specific shock

% calculate steady-state ratio

#phi_k= 1/((1-dele)/gz/gr+dele*K0K);        %KY=phi_k*XY
#QXY  = (alpha-IY)/(gz*gr/beta/(1-dele)-phi_k);
#a0   = gz*gr/beta/(1-dele)-(1-del);
#G    = (a0*QXY-alpha)/(alpha+xi*QXY);
#Phi  = 1-IY/(alpha+xi*QXY);
#del1 = alpha*(1+G)/QXY;
#QKY  = phi_k*QXY;
#phi_G= -(1-Phi)/G-1;
#CY   = 1-IY;



// now define the system

// resource constraint
CY*Ct+IY*It=Yt;

// Euler equation of investment goods producer
Pt=(1+beta)*BigOmg*gz^2*gr^2*It
   +BigOmg*gr^2*gz^2*(grt+gzt)-BigOmg*gz^2*gr^2*It(-1)
   -beta*BigOmg*gz^2*gr^2*(It(+1)+grt(+1)+gzt(+1));

// total investment 
It=alpha/(alpha+xi*QXY)*Yt
  +xi*QXY/(alpha+xi*QXY)*(xit+Qt+Xt)
  -mu*et-Pt;

// evolution of tobin's q
Qt=Lamt(+1)-Lamt
  +Qt(+1)-grt(+1)-gzt(+1)
  +beta*(1-dele)*del1*(del2)/gz/gr*ut(+1)
  +xi*G*beta*(1-dele)/gz/gr*(xit(+1)+phi_G*et(+1));

// FOC for opitmal utilization
Yt-Xt+G/(1+G)*phi_G*et=Qt+(1+del2)*ut;

// production function
Yt=alpha*(Xt+ut)+(1-alpha)*Nt;

// total capital used
Xt=(1-dele)/gz/gr*phi_k*(Kt(-1)-gzt-grt);
 
// cut-off
et=Pt-Qt;
 
// total capital stock in the end of period
Kt=-del1/phi_k*ut+(1-del)/phi_k*Xt+(1-(1-del)/phi_k)*(It-mu/phi_G*et);
 
// consumption optimal condition
Lamt=gr/(gr-beta*h)*(-gr/(gr-h)*Ct+h/(gr-h)*(Ct(-1)-grt))
    -beta*h/(gr-beta*h)*(-gr/(gr-h)*(Ct(+1)+grt(+1))+h/(gr-h)*Ct);
 
// labor supply
Lamt+Yt-Nt=psit+gamma*Nt;

// stock price
SPt=Qt+Kt;

// growth rate of output
grt=alpha/(1-alpha)*lam_zt+(lam_at+v_at-v_at(-1));

// growth rate of investment-specific technology
gzt=lam_zt;

// permanent investment-specific (Z) shock
lam_zt=rho_lam_z*lam_zt(-1)+e_lam_zt;

// permanent TFP shock
lam_at=rho_lam_a*lam_at(-1)+e_lam_at;

// transitory TFP shock
v_at=rho_v_a*v_at(-1)+e_v_at;

// labor supply shock
psit=rho_psi*psit(-1)+e_psit;

// financial shock
xit =rho_xi*xit(-1)+e_xit;

// observed variables:

// growth rate of consumption
DLogC=Ct-Ct(-1)+grt+log(gr);

// growth rate of investment
DLogI=It-It(-1)+grt+log(gr);

// growth rate of output
DLogY=Yt-Yt(-1)+grt+log(gr);

// Log of hours worked
LogN=Nt+log(1/4);

// growth rate of stock price
DLogSP=(SPt-SPt(-1))+grt+log(gr)+errt;

// growth rate of inverse investmetn goods price
DLogZP=gzt+log(gz)-(Pt-Pt(-1));

// measurement error process
errt=e_errt;

// financial index
FCt=-fc2*xit-fc3*Qt;
end;


initval;
Ct=0; It=0; Pt=0;  Qt=0; 
ut=0; Yt=0;
grt=0; gzt=0;  
Lamt=0; Nt=0; lam_zt=0; lam_at=0;  v_at=0; psit=0;
end;
%==========================================================================
shocks;
var e_lam_zt; stderr sig_lam_z;
var e_lam_at; stderr sig_lam_a;
var e_v_at  ; stderr sig_v_a;
var e_psit  ; stderr sig_psi;
var e_xit   ; stderr sig_xi;
var e_errt  ; stderr sig_err;
end;

%==========================================================================

// use estimated results to compute impulse response and variance decomposition
stoch_simul(periods=20000,order=1,hp_filter=00,irf=200,nograph);

