% This is main code to 
% compute the moments for hp filtered series (Log level, lamda=1600)
% benchmark model
% replicate "Baseline" lines in table 1 in Appendix
% six shocks
% This version: May 27, 2015

clear all
close all;

% set calibrated parameters
alpha = .30;  % capital share in production 
beta  = .99;  % discounting factor
del   = .025; % depreciation rate
dele  = .020; % exit rate 
Growgr= .42;  % growth rate of output
Growgz= 1.21; % growth rate of investment-specific shock
IY    = .2;   % Investment-output ratio
K0K   = .2;   % K0/K, capital stock of new firm to total capital stock ratio
omg   = .5;   % fraction of new firms that with bubble
gamma =  0;   % indivisible labor
theta = .9975;% expected growth rate of bubble at steady state

save Cali_Para...
alpha beta del dele theta Growgr Growgz omg gamma IY K0K;

load EstBubbleBench_results

% set parameter value with posterior mean

sig_lam_z=oo_.posterior_mode.shocks_std.e_lam_zt;
sig_lam_a=oo_.posterior_mode.shocks_std.e_lam_at;
sig_v_a  =oo_.posterior_mode.shocks_std.e_v_at;
sig_the  =oo_.posterior_mode.shocks_std.e_thet;
sig_psi  =oo_.posterior_mode.shocks_std.e_psit;
sig_xi   =oo_.posterior_mode.shocks_std.e_xit;

BigOmg   =oo_.posterior_mode.parameters.BigOmg;
h        =oo_.posterior_mode.parameters.h;
del2     =oo_.posterior_mode.parameters.del2;
mu       =oo_.posterior_mode.parameters.mu;
xi       =oo_.posterior_mode.parameters.xi;

rho_lam_z=oo_.posterior_mode.parameters.rho_lam_z;   % permanent z shock
rho_lam_a=oo_.posterior_mode.parameters.rho_lam_a;   % permanent a shock
rho_v_a  =oo_.posterior_mode.parameters.rho_v_a;   % transitory a shock
rho_the  =oo_.posterior_mode.parameters.rho_the;   % shock to theta
rho_psi  =oo_.posterior_mode.parameters.rho_psi;   % shock to labor supply
rho_xi   =oo_.posterior_mode.parameters.rho_xi;   % shock to collateral constraint


fc1=oo_.posterior_mode.parameters.fc1;
fc2=oo_.posterior_mode.parameters.fc2;
fc3=oo_.posterior_mode.parameters.fc3;


save Est_Para BigOmg h del2 mu theta xi...  
     rho_lam_z rho_lam_a rho_v_a rho_the rho_psi rho_xi ...
     sig_lam_z sig_lam_a sig_v_a sig_the sig_psi sig_xi ...
     fc1 fc2 fc3 ;
 

%========================Impulse Responses&Variance Decomposition===================================           
% variance decompositions for aggregate variable: 
% variables are corresponding to the nonstationary series excluding linear time trend
dynare BubbleImpBench.mod 
DelFiles
rmdir ('BubbleImpBench','s')
delete BubbleImpBench.m
delete BubbleImpBench_results.mat

n_sim=length(DLogC);

LogCt=cumsum(DLogC)+7.9673991;
[LogCt_tr, LogCt_cyc]=hpfilter(LogCt,1600);

LogIt=cumsum(DLogI)+6.921296;
[LogIt_tr, LogIt_cyc]=hpfilter(LogIt,1600);

LogYt=cumsum(DLogY)+8.268469213;
[LogYt_tr, LogYt_cyc]=hpfilter(LogYt,1600);

LogSPt=cumsum(DLogSP)-8.403126793;
[LogSPt_tr, LogSPt_cyc]=hpfilter(LogSPt,1600);

LogPt=-cumsum(DLogZP)+1.1203577;
[LogPt_tr, LogPt_cyc]=hpfilter(LogPt,1600);

[LogNt_tr, LogNt_cyc]=hpfilter(LogN,1600);

AR_DLogYt=corr(DLogY(2:end),DLogY(1:end-1));
AR_DLogIt=corr(DLogI(2:end),DLogI(1:end-1));

% compute std


std_Ct=std(LogCt_cyc);
std_It=std(LogIt_cyc);
std_SPt=std(LogSPt_cyc);
std_Yt=std(LogYt_cyc);
std_Nt=std(LogNt_cyc);
std_Pt=std(LogPt_cyc);

% compute autocorrelation 
AR_Ct=corr(LogCt_cyc(1:n_sim-1),LogCt_cyc(2:n_sim));
AR_It=corr(LogIt_cyc(1:n_sim-1),LogIt_cyc(2:n_sim));
AR_Yt=corr(LogYt_cyc(1:n_sim-1),LogYt_cyc(2:n_sim));
AR_Nt=corr(LogNt_cyc(1:n_sim-1),LogNt_cyc(2:n_sim));
AR_SPt=corr(LogSPt_cyc(1:n_sim-1),LogSPt_cyc(2:n_sim));
AR_Pt=corr(LogPt_cyc(1:n_sim-1),LogPt_cyc(2:n_sim));

% compute correlation with stock price
Co_Ct_Yt=corr(LogCt_cyc,LogYt_cyc);
Co_It_Yt=corr(LogIt_cyc,LogYt_cyc);
Co_Nt_Yt=corr(LogNt_cyc,LogYt_cyc);
Co_SPt_Yt=corr(LogSPt_cyc,LogYt_cyc);
Co_Pt_Yt=corr(LogPt_cyc,LogYt_cyc);


% % do the same work on data
LoadDataSentiFin_75

n_sim=length(DLogC);

LogC=cumsum(DLogC)+7.9673991;
[LogC_tr, LogC_cyc]=hpfilter(LogC,1600);

LogI=cumsum(DLogI)+6.921296;
[LogI_tr, LogI_cyc]=hpfilter(LogI,1600);

LogY=cumsum(DLogY)+8.268469213;
[LogY_tr, LogY_cyc]=hpfilter(LogY,1600);

LogSP=cumsum(DLogSP)-8.403126793;
[LogSP_tr, LogSP_cyc]=hpfilter(LogSP,1600);

LogP=-cumsum(DLogZP)+1.1203577;
[LogP_tr, LogP_cyc]=hpfilter(LogP,1600);

[LogN_tr, LogN_cyc]=hpfilter(LogN,1600);


AR_DLogY=corr(DLogY(2:end),DLogY(1:end-1));
AR_DLogI=corr(DLogI(2:end),DLogI(1:end-1));

std_C=std(LogC_cyc);
std_I=std(LogI_cyc);
std_SP=std(LogSP_cyc);
std_Y=std(LogY_cyc);
std_N=std(LogN_cyc);
std_P=std(LogP_cyc);

AR_C=corr(LogC_cyc(1:n_sim-1),LogC_cyc(2:n_sim));
AR_I=corr(LogI_cyc(1:n_sim-1),LogI_cyc(2:n_sim));
AR_Y=corr(LogY_cyc(1:n_sim-1),LogY_cyc(2:n_sim));
AR_N=corr(LogN_cyc(1:n_sim-1),LogN_cyc(2:n_sim));
AR_SP=corr(LogSP_cyc(1:n_sim-1),LogSP_cyc(2:n_sim));
AR_P=corr(LogP_cyc(1:n_sim-1),LogP_cyc(2:n_sim));

Co_C_Y=corr(LogC_cyc,LogY_cyc);
Co_I_Y=corr(LogI_cyc,LogY_cyc);
Co_N_Y=corr(LogN_cyc,LogY_cyc);
Co_SP_Y=corr(LogY_cyc,LogSP_cyc);
Co_P_Y=corr(LogY_cyc,LogP_cyc);

disp('-------------------------Business Cycle Moments(Model v.s. Data)--------------------------------')
disp('-----------------------------Standard Deviation-------------------- ');
disp('                   Model            Data' );
disp(['Output           ' num2str(std_Yt) '           ' num2str(std_Y)  ]);
disp(['Consumption      ' num2str(std_Ct) '           ' num2str(std_C)  ]);
disp(['Investment       ' num2str(std_It) '           ' num2str(std_I)  ]);
disp(['Labor            ' num2str(std_Nt) '           ' num2str(std_N)  ]);
disp(['Stock Price      ' num2str(std_SPt) '           ' num2str(std_SP)  ]);
disp(['Invest Price     ' num2str(std_Pt) '           ' num2str(std_P)  ]);

disp('-------------------------STD Relative to Output-------------------- ');
disp('                   Model            Data' );
disp(['Consumption      ' num2str(std_Ct/std_Yt) '           ' num2str(std_C/std_Y)]);
disp(['Investment       ' num2str(std_It/std_Yt) '           ' num2str(std_I/std_Y)]);
disp(['Labor            ' num2str(std_Nt/std_Yt) '           ' num2str(std_N/std_Y)]);
disp(['Stock Price      ' num2str(std_SPt/std_Yt) '            ' num2str(std_SP/std_Y)]);
disp(['Invest Price     ' num2str(std_Pt/std_Yt) '            ' num2str(std_P/std_Y)]);

disp('-------------------------Auto Correlation-------------------- ');
disp('                   Model            Data' );
disp(['Output           ' num2str(AR_Yt) '           ' num2str(AR_Y)]);
disp(['Consumption      ' num2str(AR_Ct) '           ' num2str(AR_C)]);
disp(['Investment       ' num2str(AR_It) '           ' num2str(AR_I)]);
disp(['Labor            ' num2str(AR_Nt) '           ' num2str(AR_N)]);
disp(['Stock Price      ' num2str(AR_SPt) '           ' num2str(AR_SP)]);
disp(['Invest Price     ' num2str(AR_Pt) '           ' num2str(AR_P)]);

disp('-------------------------Corr with Output-------------------- ');
disp('                   Model            Data' );
disp(['Consumption      ' num2str(Co_Ct_Yt) '          ' num2str(Co_C_Y)]);
disp(['Investment       ' num2str(Co_It_Yt) '          ' num2str(Co_I_Y)]);
disp(['Labor            ' num2str(Co_Nt_Yt) '          ' num2str(Co_N_Y)]);
disp(['Stock Price      ' num2str(Co_SPt_Yt) '          ' num2str(Co_SP_Y)]);
disp(['Invest Price     ' num2str(Co_Pt_Yt)  '          ' num2str(Co_P_Y)]);

