% This is dynare mod file to estimate 
% New benchmark model with financil condition index (FCI) data
% FCI=-fc1*(Bt-Kt(-1))-fc2*xit-fc3*Qt
% six data and six shocks 

% This version: May 13, 2013

var Ct It SPt Bt Qt Pt Nt ut mt Yt Xt Kt et grt gzt Lamt
    lam_zt lam_at v_at psit xit thet
    DLogC DLogI LogN DLogSP DLogY DLogZP FCt; 

varexo e_lam_zt e_lam_at e_v_at e_thet e_psit e_xit;

parameters alpha beta theta dele del del2 xi omg
           h BigOmg gamma Growgr Growgz  K0K 
           mu IY
           fc1 fc2 fc3
           rho_lam_z rho_lam_a rho_v_a rho_the rho_psi rho_xi;

// define parameters , 
load Cali_Para

set_param_value('alpha', alpha)
set_param_value('beta', beta )
set_param_value('del', del)
set_param_value('dele',dele)
set_param_value('gamma',gamma)
set_param_value('Growgr',Growgr )
set_param_value('Growgz',Growgz )
set_param_value('theta',theta)
set_param_value('IY',IY)
set_param_value('K0K',K0K)
set_param_value('omg',omg)

% parameters to be estimate,

BigOmg=0.07;    % investment adjustment cost
h     =0.5 ;    % habit formation
mu    =3.8321;  % elasticity of cdf at cut-off: Phi'/(1-Phi)*(epslon*)
del2  =9.76;    % capacity utilization parameter, del''/del'
xi    =0.289;   % financial constraint coefficient


% AR(1) persistence coef
rho_lam_z=0.9; % permanent z shock
rho_lam_a=0.9; % permanent a shock
rho_v_a=0.9;   % transitory a shock
rho_the=0.9;   % shock to prob of not burst
rho_psi=0.9;   % shock to labor supply
rho_xi =0.9;   % shock to collateral constraint

% financial condition coefficients
fc1=0; 
fc2=0;
fc3=0;
%-----------------------

model(linear);
// now define the system

% calculate steady-state ratio
#gr   = Growgr/100+1; % 1+growth rate of output
#gz   = Growgz/100+1; % 1+growth rate of investment-specific shock
#ga   = (gr-1)-alpha/(1-alpha)*(gz-1)+1;


#G    = 1/beta/(1-dele)/theta-1;
#del1 = gr*gz/(beta*(1-dele))-(1-del)-xi*G; % del'
#phi_x= alpha*(1+G)/del1;                   % XY=phi_x*(epslon*)
#phi_k= 1/((1-dele)/gz/gr+dele*K0K);        % KY=phi_k*XY
#BigPhi= 1-G/(1/IY*(phi_k-1+del)*phi_x-1);  % probability of not-investing
#BY   = IY/(1-BigPhi)-alpha-xi*phi_x;       % total bubble value to output ratio
#phi_G= -(1-BigPhi)/G-1;                    % loglinearized Gt=phi_G*(et*)
#CY   = 1-IY;                               % consumption-output ratio



// resource constraint
CY*Ct+IY*It=Yt;

// Euler equation of investment goods producer
Pt=(1+beta)*BigOmg*gz^2*gr^2*It
   +BigOmg*gr^2*gz^2*(grt+gzt)-BigOmg*gz^2*gr^2*It(-1)
   -beta*BigOmg*gz^2*gr^2*(It(+1)+grt(+1)+gzt(+1));

// total investment 
It=alpha/(alpha+xi*phi_x+BY)*Yt
  +xi*phi_x/(alpha+xi*phi_x+BY)*(xit+Qt+Xt)
  +BY/(alpha+xi*phi_x+BY)*Bt-mu*et-Pt;

// evolution of bubble
Bt=Lamt(+1)-Lamt+Bt(+1)
  +(1-beta*(1-dele)*theta)*phi_G*et(+1)
  +(1-(1-dele)*theta)/(1-dele)/theta*mt(+1);

// evolution of tobin's q
Qt=Lamt(+1)-Lamt
  +Qt(+1)-grt(+1)-gzt(+1)
  +beta*(1-dele)*del1*(del2)/gz/gr*ut(+1)
  +xi*(1/theta-beta*(1-dele))/gz/gr*(xit(+1)+phi_G*et(+1));

// FOC for opitmal utilization
Yt-Xt+(1-beta*(1-dele)*theta)*phi_G*et=Qt+(1+del2)*ut;

// production function
Yt=alpha*(Xt+ut)+(1-alpha)*Nt;

// total capital used
Xt=(1-dele)/gz/gr*phi_k*(Kt(-1)-gzt-grt);

// cut-off
et=Pt-Qt;

// total capital stock in the end of period
Kt=-del1/phi_k*ut+(1-del)/phi_k*Xt+(1-(1-del)/phi_k)*(It-mu/phi_G*et);

// consumption optimal condition
Lamt=gr/(gr-beta*h)*(-gr/(gr-h)*Ct+h/(gr-h)*(Ct(-1)-grt))
    -beta*h/(gr-beta*h)*(-gr/(gr-h)*(Ct(+1)+grt(+1))+h/(gr-h)*Ct);

// labor supply
Lamt+Yt-Nt=psit+gamma*Nt;

// stock price
# c1=phi_k*phi_x/(phi_k*phi_x+BY);
# c2=1-c1;
SPt=c1*(Qt+Kt)+c2*Bt;

// number of bubbly firm
mt=(1-dele)*theta*(mt(-1)+thet(-1));

// sentiment shock process
thet=rho_the*thet(-1)+e_thet;

// growth rate of output
grt=alpha/(1-alpha)*lam_zt+(lam_at+v_at-v_at(-1));

// growth rate of investment-specific technology
gzt=lam_zt;

// permanent investment-specific (Z) shock
lam_zt=rho_lam_z*lam_zt(-1)+e_lam_zt;

// permanent TFP shock
lam_at=rho_lam_a*lam_at(-1)+e_lam_at;

// transitory TFP shock
v_at=rho_v_a*v_at(-1)+e_v_at;

// labor supply shock
psit=rho_psi*psit(-1)+e_psit;

// financial shock
xit =rho_xi*xit(-1)+e_xit;

// observed variables:

// growth rate of consumption
DLogC=Ct-Ct(-1)+grt+log(gr);

// growth rate of investment
DLogI=It-It(-1)+grt+log(gr);

// growth rate of output
DLogY=Yt-Yt(-1)+grt+log(gr);

// Log of hours worked
LogN=Nt+log(1/4);

// growth rate of stock price
DLogSP=SPt-SPt(-1)+grt+log(gr);

// growth rate of inverse investmetn goods price
DLogZP=gzt+log(gz)-(Pt-Pt(-1));

// financial index
FCt=-fc1*(Bt-Kt(-1))-fc2*xit-fc3*Qt;

end;

initval;
Ct=0; It=0; Pt=0;  Bt =0;  Qt=0; ut=0; 
mt=0; Yt=0;  grt=0; gzt=0; Lamt=0; Nt=0; 
lam_zt=0; lam_at=0; v_at=0; psit=0; FCt=0;
end;
%==========================================================================
shocks;
var e_lam_zt; stderr 0.0;
var e_lam_at; stderr 0.0;
var e_v_at; stderr 0.0;
var e_thet; stderr 0.0;
var e_psit; stderr 0.0;
var e_xit;  stderr 0.0;
end;
%steady;
%check;
%==========================================================================
estimated_params;
// set prior for estimated parameters

// 1. structural parameters (the results are quite robust to the prior)
// investment adjustment cost
BigOmg  ,   ,.00001,10,gamma_pdf,2,2;

// habit formation
h       ,   ,.00000001,.99999999999,beta_pdf,1/3,0.235702260395516;

// utilization elasticity
del2    ,   ,gamma_pdf,1,1;

// elasticity of cdf at cut-off
mu      ,   ,gamma_pdf,2,2;

// xi financial constraint
xi,  beta_pdf, .3,.1,0,1;


// 2. parameters for shocks
// standard deviations
stderr e_lam_zt,inv_gamma_pdf, 0.01,inf;
stderr e_lam_at,inv_gamma_pdf, 0.01,inf;
stderr e_v_at  ,inv_gamma_pdf, 0.01,inf;
stderr e_thet  ,inv_gamma_pdf, 0.01,inf;
stderr e_psit  ,inv_gamma_pdf, 0.01,inf;
stderr e_xit   ,inv_gamma_pdf, 0.01,inf;

// AR(1) persistence parameters
rho_lam_z,beta_pdf,0.5,.2,0,1;
rho_lam_a,beta_pdf,0.5,.2,0,1;
rho_v_a  ,beta_pdf,0.5,.2,0,1;
rho_the  ,beta_pdf,0.5,.2,0,1;
rho_psi  ,beta_pdf,0.5,.2,0,1;
rho_xi   ,beta_pdf,0.5,.2,0,1;


// Financial Condition Index parameters

fc1      ,   ,gamma_pdf,1,1;
fc2      ,   ,gamma_pdf,1,1;
fc3      ,   ,gamma_pdf,1,1;

end;


estimated_params_init;

// set initial guesses for estimation procedure

BigOmg  ,   0.0340;
h       ,   0.5225;
mu      ,   3.7171;
del2    ,   8.7629;
xi      ,   0.1895; 

rho_lam_z,  0.3715 ; 
rho_lam_a,  0.9700 ; 
rho_v_a,    0.9652 ;
rho_the,    0.9474 ; 
rho_psi,    0.9830 ; 
rho_xi,     0.9453 ;

stderr e_lam_zt,  0.0059  ;
stderr e_lam_at,  0.0020  ;
stderr e_v_at  ,  0.0100  ;
stderr e_thet  ,  0.1840  ;
stderr e_psit  ,  0.0080  ;
stderr e_xit   ,  0.0248  ;

fc1,     0.1921; 
fc2,     1.6336; 
fc3,     0.4000; 

end;




// Use data series: 
// consumption, investment, hours, stock price, inverse investment goods price, financial condition index

varobs DLogC DLogI LogN DLogSP DLogZP FCt;

//===============begin to estimate==============================

// Some Notes about Estimation Issues
// (1). "mode_check" option make comparison between log post likelihood (green) and log likelihood kernal(blue)
// if two curves are almost overlap indicating results are not sensitive to the prior

// (2). "mh_jscale" is set to 0.5 such that the acceptance rate is about 0.25 which is theoretically optimal
// (3). according to Brooks&Gelman, our post dist constructed implied from random draws converges to a stationary distribution
// Dynare reports three moments: 80% quantile, 2nd order moments and 3rd order moments for both within group and between group
// "mh_drop=0.2": drop 20% of random draws.
// (4). we use the modes from previous estimation to speed up the procedure of finding the mode
 

estimation(datafile=LoadDataSentiFin_75,%mode_check,
           mode_compute=5,
           lik_init=2,
           mh_replic=200000,
           mh_nblocks=2,
           presample =0,
           mh_jscale=0.50,
           mh_drop=0.2, 
           order=1,mode_file=EstBubbleBenchRobust_mode);  
