%===================================================   
% This codes conduct the varaince decompositions in frequency domain
% replicates "Extended" lines in Table 3
% Extended model: six shocks 
% This version: May 27, 2015    
%===================================================        
clear all
close all

% set calibrated parameters
alpha= .30;   % capital share in production 
beta = .99;  % discounting factor
del  = .025; % depreciation rate
dele = .020; % exit rate 
Growgr=.42;   % growth rate of output
Growgz=1.21;  % growth rate of investment-specific shock

IY = .2;       % Investment-output ratio
K0K= .20;      % K0/K, capital stock of new firm to total capital stock ratio
omg  =.5;      % fraction of new firms that with bubble
gamma= 0;      % indivisible labor
theta= 0.9975;      % expected growth rate of bubble at steady state

save Cali_Para...
alpha beta del dele theta Growgr Growgz omg gamma IY K0K;


load EstBubbleExtend_results
% set parameter value with posterior mean

sig_lam_z=oo_.posterior_mode.shocks_std.e_lam_zt;
sig_lam_a=oo_.posterior_mode.shocks_std.e_lam_at;
sig_v_a  =oo_.posterior_mode.shocks_std.e_v_at;
sig_the  =oo_.posterior_mode.shocks_std.e_thet;
sig_psi  =oo_.posterior_mode.shocks_std.e_psit;
sig_xi   =oo_.posterior_mode.shocks_std.e_xit;
sig_err  =oo_.posterior_mode.shocks_std.e_errt;

BigOmg   =oo_.posterior_mode.parameters.BigOmg;
h        =oo_.posterior_mode.parameters.h;
del2     =oo_.posterior_mode.parameters.del2;
mu       =oo_.posterior_mode.parameters.mu;


xi       =oo_.posterior_mode.parameters.xi;

rho_lam_z=oo_.posterior_mode.parameters.rho_lam_z;   % permanent z shock
rho_lam_a=oo_.posterior_mode.parameters.rho_lam_a;   % permanent a shock
rho_v_a  =oo_.posterior_mode.parameters.rho_v_a;   % transitory a shock
rho_the  =oo_.posterior_mode.parameters.rho_the;   % shock to theta
rho_psi  =oo_.posterior_mode.parameters.rho_psi;   % shock to labor supply
rho_xi   =oo_.posterior_mode.parameters.rho_xi;   % shock to collateral constraint
rho_err  =0; %oo_.posterior_mode.parameters.rho_err;   % shock to collateral constraint


a1      =1*oo_.posterior_mode.parameters.a1; % Financial
a2      =1*oo_.posterior_mode.parameters.a2; % PM TFP
a3      =1*oo_.posterior_mode.parameters.a3; % baised TFP
a4      =1*oo_.posterior_mode.parameters.a4; % Tr TFP
a5      =1*oo_.posterior_mode.parameters.a5; % Labor


b1      =1*oo_.posterior_mode.parameters.b1; 
b2      =1*oo_.posterior_mode.parameters.b2;
b3      =1*oo_.posterior_mode.parameters.b3;
b4      =1*oo_.posterior_mode.parameters.b4;
b5      =1*oo_.posterior_mode.parameters.b5;

fc1      =oo_.posterior_mode.parameters.fc1;
fc2      =oo_.posterior_mode.parameters.fc2;
fc3      =oo_.posterior_mode.parameters.fc3;



save Est_Para BigOmg h del2 mu theta xi...  
     rho_lam_z rho_lam_a rho_v_a rho_the rho_psi rho_xi rho_err...
     sig_lam_z sig_lam_a sig_v_a sig_the sig_psi sig_xi sig_err...
     a1 a2 a3 a4 a5 b1 b2 b3 b4 b5 fc1 fc2 fc3; 
 

%========================Impulse Responses&Variance Decomposition===================================           
% variance decompositions for aggregate variable: 
% variables are corresponding to the nonstationary series excluding linear time trend
dynare BubbleImpExtend.mod 


DelFiles
delete BubbleImpExtend.m
delete BubbleImpExtend_results.mat
rmdir ('BubbleImpExtend','s')
%==========================
% In Dynare:
% the policy function have the form:
% yt=ghx_dr*yt(-1)+ghu_dr*ut
% the order of variables are in DR-order
% the following code transform the Dynare solution form into state-space
% form:
% yt=Mcx*xt(-1)+Mcu*ut;
% xt=Msx*xt(-1)+Msu*ut;
% where yt is control variables, xt is the state variables
% ut is the shocks with var-cov matrix Sigma*Sigma'
%==========================

n_var=M_.orig_endo_nbr; % number of original total variables


n_all=M_.endo_nbr;      % number of total variables defined by Dynare


% load coef-matrix
ghx_dr=oo_.dr.ghx;  
ghu_dr=oo_.dr.ghu; 

n_states=size(ghx_dr,2); % num of states variables
n_static=M_.nstatic; % num of static variables

ghx_dr_norm=ghx_dr(oo_.dr.inv_order_var,:);
ghu_dr_norm=ghu_dr(oo_.dr.inv_order_var,:);


% the order is DLogC DLogI DLogSP DLogY DLogZP LogN FCt
Msx=ghx_dr(n_static+1:n_static+n_states,:);
Msu=ghu_dr(n_static+1:n_static+n_states,:);

Mcx=ghx_dr_norm(27:33,:);
Mcu=ghu_dr_norm(27:33,:);


sigvec=diag([sig_lam_z sig_lam_a sig_v_a sig_the sig_psi sig_xi sig_err]');

n_u=size(ghu_dr,2);

% set numerical integral using CompEcon Toolbox


% the range of frequency is 6~32 quarters

quart_upp=8;
quart_low=32;


    dd=(2*pi/quart_upp-2*pi/quart_low)/1999;
    wvec=2*pi/quart_low:dd:2*pi/quart_upp;
    coef=dd*ones(length(wvec),1);

% Compute spectral density

for jj=1:n_u
    
    % define the std for each shock
    Sigma =diag(sigvec(jj,:));
    
    for ii=1:length(wvec);
       
        % frequency at w
        w=wvec(ii);
        
        % compute the spectral density for linearized variables
        Sp=SpectrumDSGE(w,Msx,Msu,Mcx,Mcu,Sigma);
        
        % define the spectrum of 1/(1-L)
        Sp_Lag=real(1/(1-exp(-1i*w))/(1-exp(1i*w)));
        
        % compute the spectral density for level
        SpMat(ii,:,jj)=[Sp_Lag;Sp_Lag;Sp_Lag;Sp_Lag;Sp_Lag;1;1]'.*Sp;

        
    end
    
    % integral spctral density for particular range of frequency
    Inte_SpMat(:,jj) = coef'*SpMat(:,:,jj);
end

% compute the variance
Sp_all=sum(Inte_SpMat,2)*ones(1,n_u);

% compute the percentage of fluctuations explained by each shock
VD=Inte_SpMat./Sp_all*100;

VD_SP=VD(3,:);
VD_C =VD(1,:);
VD_I =VD(2,:);
VD_Y =VD(4,:);
VD_L =VD(6,:);
% report results

disp('-------------------Variance decomposition-----------------------------------------------------------------')
disp(' Zgrowth     Agrowth     Atransitory      Theta       Labor       Collateral     MeaErr');
disp('                        Stock Price')
disp( num2str(VD_SP));
disp('                        ')
disp('                        Consumption ')
disp( num2str(VD_C ));
disp('                        ')
disp('                        Investment ')
disp( num2str(VD_I ));
disp('                        ')
disp('                        Output ')
disp( num2str(VD_Y ));
disp('                        ')
disp('                        Labor ')
disp( num2str(VD_L ));
