%===================================================   
% This codes conduct the varaince decompositions in frequency domain
% hybrid model, see Appendix F.3
% replicate results in Appendix F.3
% This version: May 27, 2015     
%===================================================        
clear all
close all


% set calibrated parameters
alpha= .30;   % capital share in production 
beta = .99;  % discounting factor
del  = .025; % depreciation rate
dele = .020; % exit rate 
Growgr=.42;   % growth rate of output
Growgz=1.21;  % growth rate of investment-specific shock

IY = .2;       % Investment-output ratio
K0K= .20;      % K0/K, capital stock of new firm to total capital stock ratio
omg  =.5;      % fraction of new firms that with bubble
gamma= 0;      % indivisible labor
theta= 0.9975;      % expected growth rate of bubble at steady state

save Cali_Para...
alpha beta del dele theta Growgr Growgz omg gamma IY K0K;


load EstBubbleHybrid_results
% set parameter value with posterior mean

sig_the  =oo_.posterior_mode.shocks_std.e_thet;
sig_err1  =oo_.posterior_mode.shocks_std.e_err1t;
sig_err2  =oo_.posterior_mode.shocks_std.e_err2t;
sig_err3  =oo_.posterior_mode.shocks_std.e_err3t;
sig_err4  =oo_.posterior_mode.shocks_std.e_err4t;

BigOmg   =oo_.posterior_mode.parameters.BigOmg;
h        =oo_.posterior_mode.parameters.h;
del2     =oo_.posterior_mode.parameters.del2;
mu       =oo_.posterior_mode.parameters.mu;


xi       =oo_.posterior_mode.parameters.xi;

rho_the   =oo_.posterior_mode.parameters.rho_the;  
rho11  =oo_.posterior_mode.parameters.rho11;
rho12  =oo_.posterior_mode.parameters.rho12;
rho13  =oo_.posterior_mode.parameters.rho13;
rho14  =oo_.posterior_mode.parameters.rho14;
rho21  =oo_.posterior_mode.parameters.rho21;
rho22  =oo_.posterior_mode.parameters.rho22;
rho23  =oo_.posterior_mode.parameters.rho23;
rho24  =oo_.posterior_mode.parameters.rho24;
rho31  =oo_.posterior_mode.parameters.rho31;
rho32  =oo_.posterior_mode.parameters.rho32;
rho33  =oo_.posterior_mode.parameters.rho33;
rho34  =oo_.posterior_mode.parameters.rho34;
rho41  =oo_.posterior_mode.parameters.rho41;
rho42  =oo_.posterior_mode.parameters.rho42;
rho43  =oo_.posterior_mode.parameters.rho43;
rho44  =oo_.posterior_mode.parameters.rho44;


a1      =1*oo_.posterior_mode.parameters.a1;
a2      =1*oo_.posterior_mode.parameters.a2;
a3      =1*oo_.posterior_mode.parameters.a3;
a4      =1*oo_.posterior_mode.parameters.a4;
a5      =1*oo_.posterior_mode.parameters.a5;
a6      =1*oo_.posterior_mode.parameters.a6;

save Est_Para BigOmg h del2 mu theta xi sig_the...
     rho_the   ...         
     rho11 rho12 rho13 rho14... 
     rho21 rho22 rho23 rho24 ...
     rho31 rho32 rho33 rho34 ...
     rho41 rho42 rho43 rho44  ...
     sig_err1 sig_err2 sig_err3 sig_err4...
     a1 a2 a3 a4 a5 a6 ; 
 

%========================Impulse Responses&Variance Decomposition===================================           
% variance decompositions for aggregate variable: 
% variables are corresponding to the nonstationary series excluding linear time trend
dynare BubbleImpHybrid.mod 
DelFiles
delete BubbleImpHybrid.m
delete BubbleImpHybrid_results.mat
rmdir('BubbleImpHybrid','s')
%==========================
% In Dynare:
% the policy function have the form:
% yt=ghx_dr*yt(-1)+ghu_dr*ut
% the order of variables are in DR-order
% the following code transform the Dynare solution form into state-space
% form:
% yt=Mcx*xt(-1)+Mcu*ut;
% xt=Msx*xt(-1)+Msu*ut;
% where yt is control variables, xt is the state variables
% ut is the shocks with var-cov matrix Sigma*Sigma'
%==========================

n_var=M_.orig_endo_nbr; % number of original total variables


n_all=M_.endo_nbr;      % number of total variables defined by Dynare


% load coef-matrix
ghx_dr=oo_.dr.ghx;  
ghu_dr=oo_.dr.ghu; 

n_states=size(ghx_dr,2); % num of states variables
n_static=M_.nstatic; % num of static variables

ghx_dr_norm=ghx_dr(oo_.dr.inv_order_var,:);
ghu_dr_norm=ghu_dr(oo_.dr.inv_order_var,:);


% the order is DLogC DLogI DLogSP DLogY DLogZP LogN
Msx=ghx_dr(n_static+1:n_static+n_states,:);
Msu=ghu_dr(n_static+1:n_static+n_states,:);

Mcx=ghx_dr_norm(27:32,:);
Mcu=ghu_dr_norm(27:32,:);


sigvec=diag([sig_the sig_err1 sig_err2 sig_err3 sig_err4 ]');

n_u=size(ghu_dr,2);

% the range of frequency is 6~32 quarters

quart_upp=6;
quart_low=32;

[wvec,coef] = qnwtrap(2000,2*pi/quart_upp,2*pi/quart_low);

% Compute spectral density

for jj=1:n_u
    
    % define the std for each shock
    Sigma =diag(sigvec(jj,:));
    
    for ii=1:length(wvec);
       
        % frequency at w
        w=wvec(ii);
        
        % compute the spectral density for linearized variables
        Sp=SpectrumDSGE(w,Msx,Msu,Mcx,Mcu,Sigma);
        
        % define the spectrum of 1/(1-L)
        Sp_Lag=real(1/(1-exp(-1i*w))/(1-exp(1i*w)));
        
        % compute the spectral density for level
        SpMat(ii,:,jj)=[Sp_Lag;Sp_Lag;Sp_Lag;Sp_Lag;Sp_Lag;1]'.*Sp;

        
    end
    
    % integral spctral density for particular range of frequency
    Inte_SpMat(:,jj) = coef'*SpMat(:,:,jj);
end

% compute the variance
Sp_all=sum(Inte_SpMat,2)*ones(1,n_u);

% compute the percentage of fluctuations explained by each shock
VD=Inte_SpMat./Sp_all*100;

% report results

VD_SP=VD(3,:);
VD_C =VD(1,:);
VD_I =VD(2,:);
VD_Y =VD(4,:);
VD_L =VD(6,:);
% report results

disp('-------------------Variance decomposition-----------------------------------------------------------------')
disp(' Senti   Err1    Err2    Err3    Err4 ');
disp('                        Stock Price')
disp( num2str(VD_SP));
disp('                        ')
disp('                        Consumption ')
disp( num2str(VD_C ));
disp('                        ')
disp('                        Investment ')
disp( num2str(VD_I ));
disp('                        ')
disp('                        Output ')
disp( num2str(VD_Y ));
disp('                        ')
disp('                        Labor ')
disp( num2str(VD_L ));
disp('                        ')
